import dayjs from 'dayjs'

/**
 * Parse the time to string
 * @param {(Object|string|number)} time
 * @param {string} cFormat
 * @returns {string | null}
 */
export function parseTime(time, cFormat) {
  if (arguments.length === 0) {
    return null
  }
  const format = cFormat || '{y}-{m}-{d} {h}:{i}:{s}'
  let date
  if (typeof time === 'object') {
    date = time
  } else {
    if (typeof time === 'string' && /^[0-9]+$/.test(time)) {
      time = parseInt(time)
    }
    if (typeof time === 'number' && time.toString().length === 10) {
      time = time * 1000
    }
    date = new Date(time)
  }
  const formatObj = {
    y: date.getFullYear(),
    m: date.getMonth() + 1,
    d: date.getDate(),
    h: date.getHours(),
    i: date.getMinutes(),
    s: date.getSeconds(),
    a: date.getDay(),
  }
  const time_str = format.replace(/{([ymdhisa])+}/g, (result, key) => {
    const value = formatObj[key]
    // Note: getDay() returns 0 on Sunday
    if (key === 'a') {
      return ['日', '一', '二', '三', '四', '五', '六'][value]
    }
    return value.toString().padStart(2, '0')
  })
  return time_str
}

/**
 * @param {number} time
 * @param {string} option
 * @returns {string}
 */
export function formatTime(time, option) {
  if (('' + time).length === 10) {
    time = parseInt(time) * 1000
  } else {
    time = +time
  }
  const d = new Date(time)
  const now = Date.now()

  const diff = (now - d) / 1000

  if (diff < 30) {
    return '刚刚'
  } else if (diff < 3600) {
    // less 1 hour
    return Math.ceil(diff / 60) + '分钟前'
  } else if (diff < 3600 * 24) {
    return Math.ceil(diff / 3600) + '小时前'
  } else if (diff < 3600 * 24 * 2) {
    return '1天前'
  }
  if (option) {
    return parseTime(time, option)
  } else {
    return (
      d.getMonth() +
      1 +
      '月' +
      d.getDate() +
      '日' +
      d.getHours() +
      '时' +
      d.getMinutes() +
      '分'
    )
  }
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function param2Obj(url) {
  const search = url.split('?')[1]
  if (!search) {
    return {}
  }
  return JSON.parse(
    '{"' +
    decodeURIComponent(search)
      .replace(/"/g, '\\"')
      .replace(/&/g, '","')
      .replace(/=/g, '":"')
      .replace(/\+/g, ' ') +
    '"}'
  )
}

// 函数防抖
export function debounce(fn, delay = 1000) {
  let timer = null
  return function (...args) {
    clearTimeout(timer)
    timer = setTimeout(() => {
      fn.call(this, ...args)
    }, delay)
  }
}

// 递归，匹配id的items后，返回列表数据
export function findItemNested(arr, itemId) {
  for (const o of arr || []) {
    if (o.id == itemId) return o
    const o_ = findItemNested(o.children, itemId)
    if (o_) return o_
  }
}

// 递归，过滤删除匹配id的items后，返回列表数据,如部门管理的删除
export function recursivefilter(arr, val) {
  return arr.filter((item) => {
    if (item.id === val) {
      return false
    }
    if (item.children && item.children.length > 0) {
      item.children = recursivefilter(item.children, val)
    }
    return true
  })
}

export function getCurTime() {
  const date = new Date()
  const Y = date.getFullYear()
  const M =
    date.getMonth() + 1 < 10 ? '0' + (date.getMonth() + 1) : date.getMonth() + 1
  const D = date.getDate() < 10 ? '0' + date.getDate() : date.getDate()
  const h = date.getHours() < 10 ? '0' + date.getHours() : date.getHours()
  const m = date.getMinutes() < 10 ? '0' + date.getMinutes() : date.getMinutes()
  const s = date.getSeconds() < 10 ? '0' + date.getSeconds() : date.getSeconds()
  return Y + M + D + h + m + s
}

export function getUrlFileName(s) {
  const data = s.split('/').reverse()[0]
  const re = data.match(/([^\&]*)/)[0]
  const re2 = re.match(/([^\?]*)/)[0]
  return re2
}

/**
 *
 * @param {*} source 接口返回的原始数据
 * @param {String} name 文件名
 * @param {String} ext 文件扩展名. 不传默认csv
 * @param {Boolean} noTime 文件名不需要时间. 默认false.
 */
export function downloadFile(source, name, ext = 'csv', noTime = false) {
  let type = ''
  if (ext === 'csv') {
    type =
      'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;charset=utf-8'
  }

  const blob = new Blob([source], {
    type,
  })

  let fileName = name
  if (!noTime) {
    const time = dayjs().format('YYYYMMDDHHmmss')
    fileName = `${name}-${time}`
  }

  const downloadElement = document.createElement('a')
  const href = window.URL.createObjectURL(blob) // 创建下载
  downloadElement.href = href
  downloadElement.download = `${fileName}.${ext}` // 下载后文件名
  document.body.appendChild(downloadElement)
  downloadElement.click() // 点击下载
  document.body.removeChild(downloadElement) // 下载完成移
  window.URL.revokeObjectURL(href) // 释放掉blob对象
}

export function deepClone(obj) {
  if (obj === null || typeof obj !== 'object') {
    return obj
  }

  if (Array.isArray(obj)) {
    const newArray = []
    for (let i = 0; i < obj.length; i++) {
      newArray[i] = deepClone(obj[i])
    }
    return newArray
  }

  const newObj = {}
  for (const key in obj) {
    if (Object.hasOwnProperty.call(obj, key)) {
      newObj[key] = deepClone(obj[key])
    }
  }

  return newObj
}

export function convertIdArrayToPath(options, idArray) {
  // 初始化一个空数组，用于存储路径
  const paths = [];

  // 遍历 idArray 中的每个 ID
  idArray.forEach(id => {
    // 遍历 options 中的每个选项
    options.forEach(option => {
      // 查找当前选项的子选项中是否有匹配的 ID
      const child = option.child.find(child => child.id === id);
      if (child) {
        // 如果找到匹配的子选项，将路径推入 paths 数组
        paths.push([option.id, child.id]);
      }
    });
  });

  return paths;
}
